/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.descriptor.repo.vcs;

import org.jfrog.common.config.diff.GenerateDiffFunction;

import java.util.Map;

/**
 * Created by michaelp on 6/28/15.
 */
@GenerateDiffFunction
public abstract class VcsProviderConfiguration {

    private final String prettyText;
    private final String refsPrefix;
    private final String repositoryDownloadUrl;
    private final String resourceDownloadUrl;
    private final Map<String, String> headers;
    private final String releaseDownloadUrl;

    VcsProviderConfiguration(String prettyText, String refsPrefix, String repositoryDownloadUrl) {
        this.prettyText = prettyText;
        this.refsPrefix = refsPrefix;
        this.repositoryDownloadUrl = repositoryDownloadUrl;
        this.resourceDownloadUrl = null;
        this.headers = null;
        this.releaseDownloadUrl = null;
    }

    VcsProviderConfiguration(String prettyText, String refsPrefix, String repositoryDownloadUrl,
            String resourceDownloadUrl) {
        this.prettyText = prettyText;
        this.refsPrefix = refsPrefix;
        this.repositoryDownloadUrl = repositoryDownloadUrl;
        this.resourceDownloadUrl = resourceDownloadUrl;
        this.headers = null;
        this.releaseDownloadUrl = null;
    }

    VcsProviderConfiguration(String prettyText, String refsPrefix, String repositoryDownloadUrl,
                                     String resourceDownloadUrl, Map<String, String> headers, String releaseDownloadUrl) {
        this.prettyText = prettyText;
        this.refsPrefix = refsPrefix;
        this.repositoryDownloadUrl = repositoryDownloadUrl;
        this.resourceDownloadUrl = resourceDownloadUrl;
        this.releaseDownloadUrl = releaseDownloadUrl;
        this.headers = headers;
    }

    public String getPrettyText() {
        return prettyText;
    }

    public String getRefsPrefix() {
        return refsPrefix;
    }

    public String getRepositoryDownloadUrl() {
        return repositoryDownloadUrl;
    }

    public String getResourceDownloadUrl() {
        return resourceDownloadUrl;
    }

    public String getReleaseDownloadUrl() {
        return releaseDownloadUrl;
    }

    public Map<String, String> getHeaders() {
        return headers;
    }
}
