/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.model.artifacts.search.propertysearch;

import org.apache.commons.lang.StringUtils;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.repo.exception.ItemNotFoundRuntimeException;
import org.artifactory.api.search.ItemSearchResult;
import org.artifactory.api.search.artifact.ArtifactSearchResult;
import org.artifactory.api.search.property.PropertySearchResult;
import org.artifactory.factory.InfoFactoryHolder;
import org.artifactory.fs.FolderInfo;
import org.artifactory.fs.ItemInfo;
import org.artifactory.repo.InternalRepoPathFactory;
import org.artifactory.repo.RepoPath;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.ui.rest.model.artifacts.search.BaseSearchResult;
import org.codehaus.jackson.annotate.JsonTypeName;

/**
 * @author Chen Keinan
 */
@JsonTypeName("property")
public class PropertyResult extends BaseSearchResult {

    private String relativePath;
    private String relativeDirPath;
    private String resultType;

    public PropertyResult() {
    }

    public PropertyResult(PropertySearchResult propertyResult,ArtifactoryRestRequest request) {
        super.setRepoKey(propertyResult.getRepoKey());
        this.relativePath = propertyResult.getRelativePath();
        String relDirPath = propertyResult.getRelDirPath();
        if (StringUtils.isBlank(relDirPath)) {
            relDirPath = StringUtils.equals(propertyResult.getName(), ".") ? "[repo]" : "[root]";
        }
        ItemInfo itemInfo = propertyResult.getItemInfo();
        if (!itemInfo.isFolder()) {
            this.resultType = "File"; //file
        } else if (".".equals(itemInfo.getRepoPath().getPath())) {
            this.resultType = "Repository"; //repository
        } else {
            this.resultType = "Directory"; //directory
        }
        this.relativeDirPath = relDirPath;
        super.setName(StringUtils.equals(propertyResult.getName(), ".") ? propertyResult.getRepoKey() : propertyResult.getName());
        super.setModifiedDate(itemInfo.getLastModified());
        super.setModifiedDate(itemInfo.getLastModified());
        super.setModifiedString(ContextHelper.get().getCentralConfig().format(getModifiedDate()));
        this.repoPath = InfoFactoryHolder.get().createRepoPath(propertyResult.getRepoKey(),
                propertyResult.getRelativePath(), itemInfo instanceof FolderInfo);
        setDownloadLink(request.getDownloadLink(repoPath));
        updateActions();
    }

    public String getRelativePath() {
        return relativePath;
    }

    public void setRelativePath(String relativePath) {
        this.relativePath = relativePath;
    }

    public String getRelativeDirPath() {
        return relativeDirPath;
    }

    public void setRelativeDirPath(String relativeDirPath) {
        this.relativeDirPath = relativeDirPath;
    }

    public String getResultType() {
        return resultType;
    }

    public void setResultType(String resultType) {
        this.resultType = resultType;
    }

    @Override
    protected void updateActions() {
        super.updateActions();
        if (resultType.equals("Directory") || resultType.equals("Repository")) {
            getActions().remove("Download");
            if (resultType.equals("Repository")) {
                getActions().remove("Delete");
            }
        }
    }

    @Override
    public ItemSearchResult getSearchResult() {
        ItemInfo itemInfo;
        RepoPath repoPath = InternalRepoPathFactory.create(getRepoKey(), getRelativePath());
        try {
            itemInfo = ContextHelper.get().getRepositoryService().getItemInfo(repoPath);
        } catch (ItemNotFoundRuntimeException e) {
            itemInfo = getItemInfo(repoPath);
        }
        return new ArtifactSearchResult(itemInfo);
    }
}
